

#include "audio.h"


//
// MIDAS Digital Audio System Driver
//



MIDAS_Audio::MIDAS_Audio()
{
  vol = 1.0f;
  module = 0;
  loaded = false;
  playing = false;
}


MIDAS_Audio::~MIDAS_Audio()
{
  shutdown();
}


void MIDAS_Audio::init()
{
  if ( !MIDASstartup() )
    throw Audio_Error( MIDASgetErrorMessage(MIDASgetLastError()) );

  if ( !MIDASinit() )
    throw Audio_Error( MIDASgetErrorMessage(MIDASgetLastError()) );
}


void MIDAS_Audio::shutdown()
{
   close();
   MIDASclose();  
}

void MIDAS_Audio::open( char *filename )
{
  stop();
  close();
		    
  if ( (module = reinterpret_cast<MIDASmodule*>(MIDASloadModule( filename ))) == 0 )
    throw Audio_Error( MIDASgetErrorMessage(MIDASgetLastError()) );

  if ( (handle = MIDASplayModule( module, TRUE )) == 0 )
    throw Audio_Error( MIDASgetErrorMessage(MIDASgetLastError()) );

  loaded = true;
}


void MIDAS_Audio::close()
{
  if ( loaded ) {
    MIDASfreeModule(module);
    loaded = false;
  }
}


void MIDAS_Audio::start()
{ 
  if ( !playing ) {
    if ( !MIDASstartBackgroundPlay(50) )
      throw Audio_Error( MIDASgetErrorMessage(MIDASgetLastError()) );
    playing = true;
  }
}


void MIDAS_Audio::stop()
{
  if ( playing ) {
    MIDASstopBackgroundPlay();
    MIDASstopModule( handle );
    playing = false;
  }
}


int MIDAS_Audio::order() const
{
  MIDASgetPlayStatus( handle, &status );
  return status.position;
}


int MIDAS_Audio::row() const
{
  MIDASgetPlayStatus( handle, &status );
  return status.row;
}


void MIDAS_Audio::jump_to( int pos )
{
  MIDASsetPosition( handle, pos );
}



void MIDAS_Audio::set_volume( float level ) 
{
  vol = level;
  MIDASsetMusicVolume( handle, (unsigned)(level*64.0f) );
}


float MIDAS_Audio::volume() const 
{
  return vol;
}




ostream &MIDAS_Audio::info( ostream& os ) const
{
  os << "MIDAS, " << MIDASgetOption( MIDAS_OPTION_MIXRATE ) << "Hz, ";
					   
  int mode = MIDASgetOption( MIDAS_OPTION_OUTPUTMODE );
  os << ((mode & MIDAS_MODE_8BIT) ? "8" : "16") << "-bit, ";
  os << ((mode & MIDAS_MODE_MONO) ? "'mono" : "stereo");
  os << ", volume=" << vol*100 << "%";

  os << ", loaded=" << loaded << ", playing=" << playing;

  return os;
}

