// RCS:$Id$, -*- C++ -*-
//
// Audio.h, Copyleft 1999-2000, Aspirine D-Zign Labs Inc.
// brioche/aspirine <brioche@linuxbe.org>
//
// Portable audio driver wrapper
//

#ifndef __AUDIO_H_INCLUDED
#define __AUDIO_H_INCLUDED

#include <iostream>
#include <midasdll.h>

#include "stream_fmt.h"



// Exception
class Audio_Error {

public:

  char* message;

  Audio_Error( char *msg ) : message(msg) {}

};




// Audio driver interface
class Audio {

public:

  virtual ~Audio() {};

  virtual void init() = 0;
  virtual void shutdown() = 0;

  virtual void open( char* filename ) = 0;
  virtual void close() = 0;
  
  virtual void start() = 0;
  virtual void stop() = 0;
  
  virtual int order() const = 0;
  virtual int row() const = 0;
  
  virtual void jump_to( int pos ) = 0;

  // level [0.0;1.0]
  virtual void set_volume( float level ) = 0;
  virtual float volume() const = 0;

  virtual ostream& info( ostream& os = std::cerr ) const = 0;
  
};


class NULL_Audio : public Audio {
  
public:

  NULL_Audio() { };
  ~NULL_Audio() { };

  void init() { };
  void shutdown() { };

  void open(char* filename ) { };
  void close() { };
  
  void start() { };
  void stop() { };
  
  int order() const { return 0; }
  int row() const { return 0; }
  
  void jump_to( int pos ) { };

  void set_volume( float level ) { };
  float volume() const { return 0.0; }

  ostream& info( ostream& os = std::cerr ) const { os << "NULL_Audio"; }

};




// Audio driver implementation - Uses MIDAS 1.1.1 for GNU/Linux
class MIDAS_Audio : public Audio {
  
  MIDASmodule* module;
  MIDASmodulePlayHandle handle;

  mutable MIDASplayStatus status;
  float vol;                              // Volume is cached

  bool loaded;
  bool playing;

public:

  MIDAS_Audio();
  ~MIDAS_Audio();

  void init();
  void shutdown();

  void open( char* filename );
  void close();
  
  void start();
  void stop();
  
  int order() const;
  int row() const;
  
  void jump_to( int pos );

  void set_volume( float level );
  float volume() const;

  ostream& info( ostream& os = std::cerr ) const;

};


#endif // __AUDIO_H_INCLUDED

