#include <memory.h>
#include <stdio.h>
#include <stdlib.h>

#include "pcx.h"

/*
 *	ZSoft PCX version 5.0 picture loader (handles only 8-bpp indexed pics) 
 */

//#pragma pack
static struct {	// size is 128 bytes
	char id, version, use_rle, bpp;
	short top_left_x, top_left_y, bot_right_x, bot_right_y;
	short horz_res, vert_res;
	char old_palette[48];
	char reserved;
	char bit_planes;
	short scanline_width;
	short palette_type;
	char filler[58];
} pcx_header;

int loadPcx( const char *filename, PCX_8bpp *image)
{
	FILE *fd;

	char *rle_out;
	char *rle_end;
	char dummy;
	int c;

	if(!(fd=fopen(filename, "rb"))) return -1;
		fread(&pcx_header, 1, 128, fd);

	// this loader is definetly not universal :)
	if ( ( pcx_header.id != 0x0a ) || 
	     ( pcx_header.version != 0x05 ) ||
	     ( pcx_header.bpp != 8 ) ||
	     ( pcx_header.use_rle == 0 ) ) {
			fclose(fd);
		return -1;
	}

	image->width = pcx_header.bot_right_x-pcx_header.top_left_x+1;
	image->height = pcx_header.bot_right_y-pcx_header.top_left_y+1;

	image->pixmap = (char*)malloc (image->width*image->height);
	if (!image->pixmap) return -1; // erreur d'allocation

	// unpack run length encoded pixel data 
	rle_out = image->pixmap;
	rle_end = rle_out+image->width*image->height;

	while ( rle_out < rle_end ) {
	
		char rle_cnt, rle_pix;
		fread(&rle_cnt, 1, 1, fd);

		if ( ( rle_cnt & 0xc0 ) == 0xc0 ) 
		{
			rle_cnt &= ~0xc0;
			fread(&rle_pix, 1, 1, fd);
			memset( rle_out, rle_pix, rle_cnt );
			rle_out += rle_cnt;
		} else	
			*rle_out++ = rle_cnt;
	}	

	// skip dummy 0xc0 byte 
	fread(&dummy, 1, 1, fd);

	// load rgb colormap
	for ( c = 0; c < 256; c++ ) 
	{	
		unsigned char rgb[3];
		fread(&rgb, 1, 3, fd);
		image->palette[c] = (rgb[0]<<16) + (rgb[1]<<8) + rgb[2];	
	}

	fclose (fd);
	return 0;
}
